/*----------------------------------------------------------------------------
@(#)O  UTILITY.C     Revision: 1.0-01     6/04/90
@(#)P  General utilities for Screen I/O
@(#)Q  Tim Brown - Thanks to Jeff Biery
----------------------------------------------------------------------------*/

#include <bios.h>
#include <dos.h>
#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>

union REGS regs;

long scn_mem=0xb0000000; /* mono */
int last=7;

/*----------------------------------------------------------------------------
GET CHARACTER (ROW, COLUMN)
----------------------------------------------------------------------------*/
char gch(int row,int col)
{
   return((char)*((char far *)scn_mem+row*160+col*2));
}

/*----------------------------------------------------------------------------
SET SCREEN MEMORY (TYPE) 0=MONO, 1=COLOR
----------------------------------------------------------------------------*/
void screen(int type)
{
   if (type==1)
      scn_mem=0xb8000000;
   else
      scn_mem=0xb0000000;
}


/*----------------------------------------------------------------------------
DISPLAY A STRING (ROW, COL, ATT, STR)
----------------------------------------------------------------------------*/
void atput(int row,int col,unsigned att,char *str) 
{
   register char far *ptr=(char far *)scn_mem+row*160+col*2;

   while (*str) {
      *ptr++=*(str++);
      *ptr++=att;
      }
}

/*----------------------------------------------------------------------------
DISPLAY A MENU (ROW, COL, ATT, MENU)
----------------------------------------------------------------------------*/
void printmenu(int row,int col,unsigned att,char *s[])
{
   int i=0;

   while (s[i][0]!=0) atput(row+i-1,col,att,s[i++]);
}

/*----------------------------------------------------------------------------
DISPLAY A FORMATTED STRING (ROW, COLUMN, ATT, FORMAT, PARAMS)
----------------------------------------------------------------------------*/
void dprintf(att,format,a1,a2,a3,a4,a5,a6,a7,a8,a9,a10,a11,a12,a13,a14,
            a15,a16,a17,a18,a19,a20 )
unsigned att;
char *format;
unsigned a1,a2,a3,a4,a5,a6,a7,a8,a9,a10,a11,a12,a13,a14,a15,a16,a17,a18,a19,a20;
{
   static int row=1,col=1;
	char *buf;
   register char far *ptr=(char far *)scn_mem+row*160+col*2;

	if((buf=calloc(1,200))==NULL) return;

	sprintf(buf,format,a1,a2,a3,a4,a5,a6,a7,a8,a9,a10,a11,a12,a13,a14,a15,a16,
            a17,a18,a19,a20);

   while (*buf) {
      *ptr++=*(buf++);
      if (*buf=='\n' || col>80)
         {
         col=1;
         row++;
         if (row>25)
            row=1;
         ptr=(char far *)scn_mem+row*160+col*2;
         }
      *ptr++=att;
      }

	free(buf);
}

/*----------------------------------------------------------------------------
CLEAR SCREEN (ATT)
----------------------------------------------------------------------------*/
void cls(int att)
{
   regs.h.al=0;
   regs.h.ch=0;
   regs.h.dh=24;
   regs.h.cl=0;
   regs.h.dl=79;
   regs.h.bh=att;
   regs.h.ah=6;

   int86(0x10,&regs,&regs);
}

/*----------------------------------------------------------------------------
MOVE THE CURSOR (ROW, COL)
----------------------------------------------------------------------------*/

void curset(int row,int col)
{
   regs.h.dl=col;
   regs.h.dh=row;
   regs.h.bh=0;
   regs.h.ah=2;
   int86(0x10,&regs,&regs);
}

/*----------------------------------------------------------------------------
SCROLL WINDOW UP (LINES, TOP ROW, BOT ROW, LEFT COL, RIGHT COL, ATT)
----------------------------------------------------------------------------*/

void upscroll(int lines,int urow,int lrow,int lcol,int rcol,unsigned att)
{
   regs.h.al=lines;
   regs.h.ch=urow;
   regs.h.dh=lrow;
   regs.h.cl=lcol;
   regs.h.dl=rcol;
   regs.h.bh=att;
   regs.h.ah=6;

   int86(0x10,&regs,&regs);
}

/*----------------------------------------------------------------------------
SCROLL WINDOW DOWN (LINES, TOP ROW, BOT ROW, LEFT COL, RIGHT COL, ATT)
----------------------------------------------------------------------------*/

void dnscroll(int lines,int urow,int lrow,int lcol,int rcol,unsigned att)
{
   regs.h.al=lines;
   regs.h.ch=urow;
   regs.h.dh=lrow;
   regs.h.cl=lcol;
   regs.h.dl=rcol;
   regs.h.bh=att;
   regs.h.ah=7;

   int86(0x10,&regs,&regs);
}

/*----------------------------------------------------------------------------
DISPLAY DATE (ROW, COLUMN, ATT)
----------------------------------------------------------------------------*/

void dateat(int row, int col,unsigned att)
{
   char str[10];
   int x;

   regs.h.ah=42;
   int86(0x21,&regs,&regs);
   sprintf(str,"%02d-%02d-%02d",regs.h.dh,regs.h.dl,regs.x.cx-1900);
   atput(row,col,att,str);
}

/*----------------------------------------------------------------------------
DISPLAY TIME (ROW, COL, ATT)
----------------------------------------------------------------------------*/

void timeat(int row, int col, unsigned att)
{
   char str[10];
   int x;

   regs.h.ah=44;
   int86(0x21,&regs,&regs);
   sprintf(str,"%02d:%02d:%02d",regs.h.ch,regs.h.cl,regs.h.dh);
   atput(row,col,att,str);
}

/*----------------------------------------------------------------------------
DISPLAYS A CHARACTER (ROW, COLUMN, ATT, CHAR)
----------------------------------------------------------------------------*/
void pch(int row,int col,unsigned att,char c)
{
   *((char far *)scn_mem+row*160+col*2)=c;
   *((char far *)scn_mem+row*160+col*2+1)=att;
}

/*----------------------------------------------------------------------------
DRAWS A HORZ LINE (ROW, COLUMN, ATT, LENGTH)
----------------------------------------------------------------------------*/

void hline(int row,int col,unsigned att,int len)
{
   int x;
   for (x=0;x<len;x++) pch(row,col++,att,196);
}

/*----------------------------------------------------------------------------
DRAWS A VERTICLE LINE (ROW, COLUMN, ATT, LENGTH)
----------------------------------------------------------------------------*/

void vline(int row,int col,unsigned att,int len)
{
   int x;
   for (x=0;x<len;x++) pch(row++,col,att,179);
}

/*----------------------------------------------------------------------------
DRAW A SINGLE LINE BOX (ROW, COL, HORZ LEN, VERT LEN, ATT)
----------------------------------------------------------------------------*/

void box(int row,int col,int hlen,int vlen,unsigned att)
{
   int x,cols=col,rows=row;

   hlen--; vlen--;

   for (x=0;x<vlen;x++) {pch(row,col,att,179); pch(row++,col+hlen,att,179);}
   for (x=0;x<hlen;x++) {pch(row,col,att,196); pch(row-vlen,col++,att,196);}
   pch(rows,cols,att,218);
   pch(rows,cols+hlen,att,191);
   pch(rows+vlen,cols,att,192);
   pch(rows+vlen,cols+hlen,att,217);
}

/*----------------------------------------------------------------------------
GET KEYBOARD INPUT WITH EXTENDED CODES (SEE KEYS.H)
RETURNS KEY CODE.
----------------------------------------------------------------------------*/
unsigned getkey()
{
   unsigned key=_bios_keybrd(_KEYBRD_READ),
            scan=(key&0xFF00)>>8,
            ascii=(char)(key&0xFF);

   return (ascii)?ascii:scan+255;
}

/*----------------------------------------------------------------------------
DISPLAY A BAR (ROW, COLUMN, ATT, LENGTH)
----------------------------------------------------------------------------*/
void bar(int row,int col,unsigned att,int length)
{
   while (length--) *((char far *)scn_mem+row*160+(col++)*2+1)=att;
}

